<?php

namespace Bydedi\Utils\Image;

use Exception;
use Imagine\Exception\InvalidArgumentException;
use Imagine\Image\Box;
use Imagine\Image\ImageInterface;
use Imagine\Image\ImagineInterface;
use Imagine\Image\Point;
use Psr\Log\LoggerInterface;

class Image
{
    /**
     * @var \Imagine\Image\ImagineInterface
     */
    protected $imagine;

    protected $paths;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    public function __construct(ImagineInterface $imagine, $paths, LoggerInterface $logger = null)
    {
        $this->imagine = $imagine;
        $this->paths = $paths;
        $this->logger = $logger;
    }

    public function transform($path, $options = array())
    {

        $infos_path = parse_url($path);
        $src = $this->paths['web'] . $infos_path['path'];

        $imagename = substr($src, strrpos($src, '/') + 1);
        $imagename = substr($imagename, 0, strrpos($imagename, '.'));

        $imagename = strtolower(preg_replace('/[^a-zA-Z0-9_\.]/', '_', $imagename));

        if (!empty($options['resize']['width'])) {
            $options['resize']['height'] = $options['resize']['height'] > 0 ?
                $options['resize']['height'] : $options['resize']['width'];
            $imagename .= '_' . $options['resize']['width'] . 'x' . $options['resize']['height'];
        }

        $dirDest = str_replace($this->paths['web'], '', $src);
        $dirDest = str_replace(basename($src), '', $dirDest);
        $dirDest = $this->paths['web'] . '/media/cache' . $dirDest;

        $imagename .= substr($src, strrpos($src, '.')); // add extension

        $newPath = $dirDest . $imagename;

        if (file_exists($newPath) && filemtime($newPath) > filemtime($src)) {
            return str_replace($this->paths['web'], 'http://' . $_SERVER['HTTP_HOST'], $newPath);
        }

        try {
            $image = $this->imagine->open($src);
        } catch (Exception $e) {
            return '/media/blank.gif';
        }

        if (array_key_exists('resize', $options)) {
            $size = new Box($options['resize']['width'], $options['resize']['height']);
            $mode = ImageInterface::THUMBNAIL_INSET;
            $resizeimg = $image->thumbnail($size, $mode);

            $sizeR     = $resizeimg->getSize();
            $widthR    = $sizeR->getWidth();
            $heightR   = $sizeR->getHeight();

            $image  = $this->imagine->create($size);
            $startX = $startY = 0;
            if ($widthR < $options['resize']['width']) {
                $startX = ( $options['resize']['width'] - $widthR ) / 2;
            }
            if ($heightR < $options['resize']['height']) {
                $startY = ( $options['resize']['height'] - $heightR ) / 2;
            }
            $image->paste($resizeimg, new Point($startX, $startY));
        }

        if (array_key_exists('watermark', $options)) {
            $watermark = $this->imagine->open($options['watermark']);

            $size      = $image->getSize();
            $wSize     = $watermark->getSize();

            $bottomRight = new Point($size->getWidth() - $wSize->getWidth(), $size->getHeight() - $wSize->getHeight());

            $image->paste($watermark, $bottomRight);
        }

        if (!is_dir($dirDest) && !@mkdir($dirDest, 0775, true)) {
            throw new Exception(
                'Cannot create folder. Permission denied'
            );
        }
        if (!is_writable($dirDest) && !@chmod($dirDest, 0775)) {
            throw new Exception(
                'Cannot write to folder. Permission denied'
            );
        }

        $image->save($newPath);

        return str_replace($this->paths['web'], '', $newPath);
    }
}
